import { Component, OnInit, NgModule, Inject, Input } from '@angular/core';
import { MatTableDataSource, MatDialog, MatDialogRef, MAT_DIALOG_DATA, MatDialogModule, MatDialogConfig } from '@angular/material';

import { Location } from '@angular/common';
// import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable, Subject } from 'rxjs';
import { Subscription } from 'rxjs';
import { fromEvent } from 'rxjs';
import { timer } from 'rxjs/observable/timer';
import { repeat, takeUntil, skip } from 'rxjs/operators';
import { Router, NavigationStart } from '@angular/router';

import {Strudelapp} from '../strudelapp';
import { StrudelappsService } from '../strudelapps.service';
import { TesService } from '../tes.service';
import {BackendSelectionService } from '../backend-selection.service';
import { AuthorisationService } from '../authorisation.service';
import { Identity } from '../identity';
import { Computesite, Health } from '../computesite';
import { LogoutdialogComponent } from '../logoutdialog/logoutdialog.component';

import { SshAuthzServer } from '../identity';
import { ComputesitesService } from '../computesites.service';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';




export interface Quota {
  resource: string;
  usage: string;
  quota: string;
}




@Component({
  selector: 'app-launcher',
  templateUrl: './launcher.component.html',
  styleUrls: ['./launcher.component.css']
})
export class LauncherComponent implements OnInit {
  public strudelapps: Strudelapp[];
  @Input() menuopen: boolean;

  public appSubject: BehaviorSubject<Strudelapp>;
  public authorised: boolean;
  //public identity: Identity;
  public identitySubject: BehaviorSubject<Identity>;

  public identities: Identity[];
  public sshauthzservers: SshAuthzServer[];
  private launchwindow: any;
  private launchwindowWatcher: any;
  private subscriptions: Subscription[];
  //public quotas: BehaviorSubject<any[]>;
  displayedColumns: string[] = [];


  constructor( public dialog: MatDialog,
                public tesService: TesService,
                public backendSelectionService: BackendSelectionService,
                public authService: AuthorisationService,
                public computeSitesService: ComputesitesService,
                private router: Router,
                ) {
      this.subscriptions = [];
      this.subscriptions.push(this.authService.sshAuthzServers.subscribe(o => {this.updateSshAuthZServers(o)}));
      this.subscriptions.push(this.computeSitesService.appidentities.subscribe(o => this.navLogin(o)));
      this.subscriptions.push(this.computeSitesService.appidentities.subscribe(o => this.getHealth(o)));
      this.appSubject = new BehaviorSubject<Strudelapp>(null);
      this.identitySubject = new BehaviorSubject<Identity>(null);
    //this.quotas = new BehaviorSubject<any[]>([]);
  }

  countErrors(a: Health[], b: Health[]) {
    var count: number = 0
    var h: Health;
    for (h of a) {
      if (h.stat == 'error' || h.stat == 'warn') {
        count++;
      }
    }
    for (h of b) {
      if (h.stat == 'error' || h.stat == 'warn') {
        count++;
      }
    }
    return count;
  }

  calculateCols(data) {
    return data.map(function(r) { return r.key });
  }

  humanKBytes(n: number) {
    if (n > 1024*1024*1024) {
      let v = n/1024/1024/1024;
      return v.toFixed(0)+'TB';
    }
    if (n > 1024*1024) {
      let v = n/1024/1024;
      return v.toFixed(0)+'GB';
    }
    if (n > 1024) {
      let v = n/1024;
      return v.toFixed(0)+'MB';
    }
    return '0 MB';
  }

  rowClass(row) {
    if (row.stat == 'error' || row.stat == 'warn') {
      return 'health-warn';
    } else {
      return 'health-ok';
    }
  }

  navLogin(o) {
    if (o.length == 0) {
      this.router.navigate(['/login']);
    }
  }

  getHealth(o) {
    for (let id of o) {
      this.tesService.getHealthAlerts(id);
    }
  }

  updateSshAuthZServers(o) {
    this.sshauthzservers = o;
  }

  ngOnInit() {
    this.strudelapps = [];
  }

  ngOnDestroy() {
    for (let s of this.subscriptions) {
      s.unsubscribe();
    }
  }

  selectId(id: Identity) {
    this.identitySubject.next(id);
    this.appSubject.next(null);

    if (id != null) {
      for (let a of id.accountalerts.value) {
        console.log('acocunt alert');
        console.log(a);
      }
    }
    //if (id != null) {
      // We will subscribe to account alerts until the id changes (in which case we will subscribe to account alerts
      // on the next id
      // BUT
      // since identitySubject is a behaviorsubject and already has a value, we must skip(1)
    //  id.accountalerts.pipe(takeUntil(this.identitySubject.pipe(skip(1)))).subscribe((msgs) => { this.quotas.next(msgs.filter(q => q.type =='quota')) });
    //}
  }

  selectApp(app: Strudelapp) {
    this.appSubject.next(app);
  }


}
